/*
    Shiori
        あかつきみさき (みくちぃP)

    このスクリプトについて
        スクリプト・アニメーションプリセットのランチャースクリプト.
        横並び・縦並び好きな方向に切り替えることができます.
        スクリプト名を変更することで複数起動することができます.

    使用方法
        ScriptUI Panelsフォルダに入れてから,ウィンドウより実行.

    動作環境
        Adobe After Effects CS6以上

    バージョン情報
        2018/07/01 Ver 1.0.0 Release
*/

(function () {
    var data = {
        name: "Shiori",
        version: "1.0.0"
    };

    Settings.prototype.initSetting = function (sectionName, keyName, defauleValue) {
        if (!app.settings.haveSetting(sectionName, keyName)) {
            app.settings.saveSetting(sectionName, keyName, defauleValue);
        }

        return app.settings.getSetting(sectionName, keyName);
    }

    var FOLDER_PATH = new Folder(app.settings.initSetting($.fileName, "FOLDER_PATH", new Folder(new File($.fileName).parent.parent).fsName));
    var ORIENTATION_ROW = Number(app.settings.initSetting($.fileName, "ORIENTATION_ROW", 0));

    var btnFiles = [];
    if ((FOLDER_PATH != null) && FOLDER_PATH.exists) {
        btnFiles = FOLDER_PATH.getFiles(/.jsx(bin)?$|.ffx$/);
    }

    var executeFile = function (targetFile) {
        var targetFile = new File(targetFile);
        if (targetFile == null) {
            return 0;
        }

        if (targetFile.fsName.match(/.ffx$/)) {
            var actComp = app.project.activeItem;
            if (actComp == null || !(actComp instanceof CompItem)) {
                return 0;
            }

            var selLayers = actComp.selectedLayers;
            if (selLayers.length == 0 || ScriptUI.environment.keyboardState.ctrlKey) {
                app.beginUndoGroup(targetFile.name);
                var layerName = decodeURIComponent(targetFile.name).split(".");
                layerName.pop();
                var addedLayer = actComp.layers.addSolid([0, 0, 0], layerName.join("."), actComp.width, actComp.height, actComp.pixelAspect, actComp.duration);
                addedLayer.applyPreset(targetFile);
                app.endUndoGroup();
                return 0;
            }

            app.beginUndoGroup(targetFile.name);
            for (var i = 0; i < selLayers.length; i++) {
                selLayers[i].applyPreset(targetFile);
            }
            app.endUndoGroup();
        } else if (targetFile.fsName.match(/.jsx(bin)?$/)) {
            $.evalFile(targetFile);
        }
    }

    var changeFolder = function () {
        if (ScriptUI.environment.keyboardState.ctrlKey) {
            FOLDER_PATH.execute();
            return 0;
        }

        FOLDER_PATH = new Folder(FOLDER_PATH).selectDlg("Select .jsx(bin) & .ffx folder");

        if (FOLDER_PATH == null) {
            return 0;
        }

        btnFiles = [];
        btnFiles = FOLDER_PATH.getFiles(/.jsx(bin)?$|.ffx$/);
        app.settings.saveSetting($.fileName, "FOLDER_PATH", FOLDER_PATH.fsName);
    }

    function buildUI(thisObj) {
        var shioriWin;

        if (thisObj instanceof Panel) {
            shioriWin = thisObj;
        } else {
            shioriWin = new Window("palette", data.name + " " + data.version, undefined, { resizeable: true });
        }

        shioriWin.spacing = 1;
        shioriWin.margins = [0, 0, 0, 0];
        shioriWin.iconBtns = [];
        shioriWin.size = [400, 500];

        shioriWin.removeAllButton = function () {
            if (!this.btnGrp) {
                return;
            }

            for (var i = this.btnGrp.children.length; i > 0; i--) {
                this.btnGrp.remove(0);
            }

            this.remove(0);
        }

        shioriWin.reload = function () {
            shioriWin.btnGrp = shioriWin.add("group");
            shioriWin.btnGrp.spacing = 0;
            shioriWin.btnGrp.margins = [0, 0, 0, 0];
            shioriWin.btnGrp.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];

            if (btnFiles.length == 0) {
                var st = shioriWin.btnGrp.add("statictext", undefined, "No files");
                st.justify = "center";
                st.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
            }

            var btns = [];

            for (var i = 0; i < btnFiles.length; i++) {
                btns[i] = shioriWin.btnGrp.add("iconbutton");
                btns[i].fileObj = btnFiles[i];
                btns[i].helpTip = decodeURIComponent(btnFiles[i].name);
                btns[i].alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];

                btns[i].text = decodeURIComponent(btnFiles[i].name);

                btns[i].onClick = function () {
                    executeFile(new File(this.fileObj));
                }
            }

            btns[btnFiles.length] = shioriWin.btnGrp.add("group");
            btns[btnFiles.length].spacing = 0;
            btns[btnFiles.length].margins = [0, 0, 0, 0];
            btns[btnFiles.length].orientation = "row";

            if (ORIENTATION_ROW == 0) {
                btns[btnFiles.length].alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.BOTTOM];
                btns[btnFiles.length].minimumSize.height = 25;
                btns[btnFiles.length].maximumSize.height = 25;
            } else {
                btns[btnFiles.length].alignment = [ScriptUI.Alignment.RIGHT, ScriptUI.Alignment.FILL];
                btns[btnFiles.length].minimumSize.width = 75;
                btns[btnFiles.length].maximumSize.width = 75;
            }

            var folderBtn = btns[btnFiles.length].add("button");
            folderBtn.text = "Folder";
            folderBtn.helpTip = "Change folder";
            folderBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
            folderBtn.onClick = function () {
                changeFolder();

                shioriWin.removeAllButton();
                shioriWin.reload();
                shioriWin.layout.layout(true);
                shioriWin.layout.resize();
            }

            var orientationBtn = btns[btnFiles.length].add("button");
            orientationBtn.helpTip = "Change orientation";
            orientationBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
            orientationBtn.onClick = function () {
                app.settings.saveSetting($.fileName, "ORIENTATION_ROW", 1 - Number(ORIENTATION_ROW));
                ORIENTATION_ROW = Number(app.settings.getSetting($.fileName, "ORIENTATION_ROW"));

                shioriWin.removeAllButton();
                shioriWin.reload();
                shioriWin.layout.layout(true);
                shioriWin.layout.resize();
            }

            var helpBtn = btns[btnFiles.length].add("button");
            helpBtn.text = "?";
            helpBtn.helpTip = "?";
            helpBtn.alignment = [ScriptUI.Alignment.FILL, ScriptUI.Alignment.FILL];
            helpBtn.onClick = function () {
                alert("Shiori\nMisaki Akatsuki (SUNRISE MOON)\n\nScript & animation preset launcher.", "Shiori");
            }

            if (ORIENTATION_ROW == 0) {
                orientationBtn.text = "---";
                shioriWin.btnGrp.orientation = "column";
                folderBtn.text = "Folder";
            } else {
                orientationBtn.text = "|";
                shioriWin.btnGrp.orientation = "row";
                folderBtn.text = "F";
            }
        }

        shioriWin.reload();
        shioriWin.onResizing = shioriWin.onResize = function () {
            this.layout.resize();
        }
        shioriWin.layout.layout();
        shioriWin.layout.resize();

        if (shioriWin instanceof Window) {
            shioriWin.center();
            shioriWin.show();
        }

        return shioriWin;
    }

    buildUI(this);
    return 0;
}).call(this);
